'use strict';

var fs = require('fs');
var path = require('path');
var logger = require('jsdoc/util/logger'); // eslint-disable-line

var allDoclets = {};
var testDoclets = {};

exports.handlers = {
	newDoclet: function newDoclet(e) {
		// Set contants.
		var meta = e.doclet.meta;
		var description = e.doclet.description;

		// Save doclets.
		if (typeof description === 'string' && meta.path && meta.filename) {
			allDoclets[e.doclet.longname] = e;
		}
	},

	parseComplete: function parseComplete() {
		var errorMsg = [];
		var testKeys = Object.keys(testDoclets).sort();

		// Add descriptions
		for (var i in testDoclets) {
			// eslint-disable-line no-restricted-syntax
			var testDescription = [];
			var meta = testDoclets[i].meta;

			// Set sourceString
			var fileContent = fs.readFileSync(path.join(meta.path || '.', meta.filename), 'utf-8');

			// Set sourceLines
			var srcString = '';
			for (var j = meta.range[0]; j < meta.range[1]; j += 1) {
				// eslint-disable-line no-restricted-syntax
				srcString += fileContent[j];
			}

			var srcArr = srcString.split('\n');
			for (var _j in srcArr) {
				// eslint-disable-line no-restricted-syntax
				if (/^\s*it\s*\(/.test(srcArr[_j])) {
					testDescription.push(srcArr[_j].replace(/^\s*it\s*\(\s*('|")/, '') // remove before.
					.replace(/('|").*$/, '') // remove after
					);
				}
			}

			// Set new description if commentsArr is full
			var description = '';
			if (testDescription.length > 0) {
				description = '\n<b>Tests:</b>\n<ol>\n';
				testDescription.forEach(function (test) {
					description += '<li>' + test + '</li>\n';
				});
				description += '</ol>';
			}

			try {
				allDoclets[i].doclet.description += description;
			} catch (err) {
				errorMsg.push(meta.path + ' ' + meta.filename + ' on line ' + meta.lineno);
			}
		}

		// Log wrong pointers
		if (errorMsg.length > 0) {
			logger.error('Tag "@test" wrong pointer:\n' + errorMsg.join('\n'));
			errorMsg = [];
		}

		// Test coverage
		Object.keys(allDoclets).forEach(function (name) {
			if (testKeys.indexOf(name) === -1) {
				errorMsg.push(' > ' + name);
			}
		});

		// Log missing tests
		if (errorMsg.length > 0) {
			logger.error('Test missing on:\n' + errorMsg.join('\n'));
		}
	}
};

exports.defineTags = function (dictionary) {
	// Define test tag.
	dictionary.defineTag('test', {
		mustHaveValue: true,
		onTagged: function onTagged(doclet, tag) {
			// Save discovered doclets.
			testDoclets[tag.value] = doclet;
		}
	});
};